@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+# !/usr/bin/env python
 # -*- coding: utf-8 -*-
 # --------------------------------------------------------------
 # ProjectName: Logcat Monitor
@@ -8,16 +8,16 @@
 # Description:
 #   This is a tool class for use cases to find exception information in logcat in real time
 # --------------------------------------------------------------
-
-# Configure keywords to be monitored
 import json
 import os
 import subprocess
 import time
 from datetime import datetime
 
+from logcat_monitor.statistics import Statistics
 from logcat_monitor.threadpool import ThreadPool
 
+# 缺省的异常捕捉关键字
 KEYWORDS_DICT = {"ANR in ": "Anr",
                  "FATAL EXCEPTION:": "Fatal",
                  "signal 6": "Signal6",
@@ -26,37 +26,64 @@ KEYWORDS_DICT = {"ANR in ": "Anr",
                  "CRASH: ": "Crash",
                  "Force Closed": "ForceClose"}
 
+# 捕捉到异常后截取的日志行数
 INTERCEPTED_MAX_ROWS = 100
 
+# 存放logcat日志文件的文件夹名
 LOG_FOLDER_NAME = "logcats"
 
-
-class Statistics:
-    def increase(self, key_dict, key):
-        """
-            Increase the number of error
-        :param key_dict:
-        :param key:
-        :return:
-        """
-        if key in key_dict:
-            attr = key_dict[key]
-            count = getattr(self, attr, 0)
-            target = count + 1
-            setattr(self, attr, target)
-            print("Count [%s] curent:%d target:%d" % (key, count, target))
+# 默认单个日志文件的大小
+DEFAULT_LOGFILE_SIZE = 1024 * 1024 * 200
 
 
 class LogcatMonitor:
-    # Control on and off
+    """
+    Logcat 监控器
+    """
+    # 控制获取logcat停止的标志位
     _stop_logcat = True
-    # logs buffer
+    # 存储日志的缓冲区
     _logs = []
     _key_dict = KEYWORDS_DICT
     _pool = ThreadPool(1)
     _save_logcat = True
+    _logfile_size = DEFAULT_LOGFILE_SIZE
+    _log_file = None
+    _write_log_file_size = 0
 
-    def __init__(self, serial_number=None, parent_folder=None, key_dict=None, rows=INTERCEPTED_MAX_ROWS):
+    def calculate_logfile_size(self, logfile_size: str):
+        """
+        根据字符串形式获得正式的文件大小数字
+        :param logfile_size:
+        :return:
+        """
+        self._logfile_size = DEFAULT_LOGFILE_SIZE
+        symbols = ('K', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y')
+        if logfile_size is None:
+            length = 0
+        else:
+            length = len(logfile_size)
+        if length > 0:
+            unit = logfile_size[length - 1:]
+            if unit.isdecimal():
+                self._logfile_size = int(logfile_size)
+            else:
+                if length == 1:
+                    value = 1
+                else:
+                    value = int("".join(list(filter(str.isdigit, logfile_size))))
+                for i, s in enumerate(symbols):
+                    if unit == s:
+                        self._logfile_size = pow(1024, i + 1) * int(value)
+                        break
+                else:
+                    print("Input log file size error, use default value")
+        print("Logfile size: %s" % self._logfile_size)
+        return self._logfile_size
+
+    def __init__(self, serial_number=None, parent_folder=None, key_dict=None,
+                 rows=INTERCEPTED_MAX_ROWS, logfile_size=None):
+        self.calculate_logfile_size(logfile_size)
         if key_dict is not None:
             for key in key_dict:
                 self._key_dict[key] = (key_dict[key])
@@ -104,6 +131,10 @@ class LogcatMonitor:
                 print(ex)
 
     def filter_file(self, file_name):
+        """
+        过滤文件
+        :param file_name: 文件名称
+        """
         intercept = False
         rows = 0
         keys = self._key_dict.keys()
@@ -112,7 +143,7 @@ class LogcatMonitor:
         with open(file_name, 'r', encoding='utf-8') as log_file:
             line = log_file.readline()
             while line:
-                line_count = line_count + 1
+                line_count += 1
                 if not intercept:
                     # Read data normally
                     for key in keys:
@@ -130,7 +161,7 @@ class LogcatMonitor:
                     if rows < self._rows:
                         # Add the current line to the logs
                         self._logs.append(line)
-                        rows = rows + 1
+                        rows += 1
                     else:
                         # Finished reading the required logs
                         intercept = False
@@ -143,6 +174,23 @@ class LogcatMonitor:
         # Save statistics
         self._save_statistics()
 
+    def create_new_logfile(self):
+        """
+        创建一个新的文件，用来保存日志
+        """
+        string_time = datetime.now().strftime("%Y%m%d%H%M%S")
+        logcat_file_name = "%s%slogcat_%s.txt" % (self._self_folder, os.sep, string_time)
+        self.close_logfile()
+        self._log_file = open(logcat_file_name, 'w+', encoding='utf-8')
+        self._write_log_file_size = 0
+
+    def close_logfile(self):
+        """
+        关闭打开的日志文件
+        """
+        if self._log_file is not None and not self._log_file.closed:
+            self._log_file.close()
+
     def _filter_keyword(self):
         """
             Keep reading logcat output, find error information.
@@ -157,15 +205,11 @@ class LogcatMonitor:
         keys = self._key_dict.keys()
         find_key = None
         # File storage location(file full path)
-        print("####### STRAT LOGCAT WRITE #######")
-        logcat_file_name = "%s%slogcat.txt" % (self._self_folder, os.sep)
+        print("####### START LOGCAT WRITE #######")
         line_count = 0
-        log_file = None
-
         # Open logcat file
         if self._save_logcat:
-            log_file = open(logcat_file_name, 'w+', encoding='utf-8')
-
+            self.create_new_logfile()
         while not self._stop_logcat:
             print("Get logcat")
             # Keep reading data
@@ -177,15 +221,20 @@ class LogcatMonitor:
                         # Stop
                         print("####### STOP LOGCAT #######")
                         break
-                    line_count = line_count + 1
+                    line_count += 1
                     try:
                         line_str = line.decode(encoding="utf-8", errors="ignore")
                     except Exception as ex:
                         print(ex)
                         line_str = str(ex)
+                    # 检查文件是否超过限定的大小
+                    if self._write_log_file_size > self._logfile_size:
+                        self.create_new_logfile()
+
                     # Write logcat file
-                    if log_file is not None and not log_file.closed:
-                        log_file.write(line_str)
+                    if self._log_file is not None and not self._log_file.closed:
+                        self._log_file.write(line_str)
+                        self._write_log_file_size += len(line_str)
 
                     if not intercept:
                         # Read data normally
@@ -204,7 +253,7 @@ class LogcatMonitor:
                         if rows < self._rows:
                             # Add the current line to the logs
                             self._logs.append(line_str)
-                            rows = rows + 1
+                            rows += 1
                         else:
                             # Finished reading the required logs
                             intercept = False
@@ -217,9 +266,7 @@ class LogcatMonitor:
                 print("End monitor _logcat")
                 sub.kill()
                 # Close logcat file
-                if log_file is not None and not log_file.closed:
-                    log_file.close()
-
+                self.close_logfile()
                 if not self._stop_logcat:
                     # Abnormal termination
                     # wait...
